#################################################
# POLS 506: Bayesian Statistics
# Dr. Justin Esarey
# Lecture 0: Introduction to R
#################################################


#####################################################################
# We will start with an introduction to the basic functions of R.
# NOTE: code preceded by a number sign (#) will not be executed by R
#####################################################################

# How to load data and run basic statistical models

rm(list=ls())       # clear out anything in memory
library(foreign)    # this loads functions that let you read Stata data sets

# Set the working directory of your R session
setwd("C:/Dropbox/jesarey_documents/POLS 506 - Topics in Methodology I/Lecture 0 - Introduction to R")

?read.dta    # what does this function do?
my.data.set<-read.dta(file="506_lecture0.dta")  # read.dta() loads the Stata data set, called lecture0.dta
# now we have my.data.set in memory... let's look at it
View(my.data.set)     # View() opens a spreadsheet that lets you see your data frame

# variables in your data set live inside of your data frame. 
happiness       # doesn't work!
my.data.set$happiness         # now it works
# if you don't want to reference the data frame every time, you can "attach" it to R's memory.
attach(my.data.set)
happiness
search()    # see what's attached

# now, let's do some analysis. What's the relationship between happiness and graduate stipend?
?plot
plot(happiness~stipend)
plot(happiness~stipend, main="Graduate Students: Happiness vs. Stipend", xlab="stipend (in thousands of $)", ylab="happiness (percentile)")
legend("topleft", legend=c("student"), pch=c(1))

# What's the relationship between happiness and year in school?
plot(happiness~years, main="Graduate Students: Happiness vs. Time", xlab="# of years in graduate school", ylab="happiness (percentile)")


# run a (somewhat inappropriate!) linear model relating happiness to stipend and years in school
?lm
linear.model<-lm(happiness~years+stipend)
# show the results
summary(linear.model)
# get the coefficients and VCV matrix
coefficients(linear.model)
vcov(linear.model)
# residuals and predicted values
resid<-residuals(linear.model)
pred<-predict(linear.model)
plot(resid~pred)
plot(resid~years)


# create a dichotomous variable called "happy"
# =1 if happiness > 0.5, =0 otherwise
detach(my.data.set)                                       # detach the data set
my.data.set$happy<-my.data.set$happiness>0.5              # add a variable that meets the specification
View(my.data.set) 
my.data.set$happy<-as.numeric(my.data.set$happy)          # change T/F to numeric
View(my.data.set)
attach(my.data.set)

plot(happy~stipend)
plot(happy~years)

# run a logit model
?glm
logit.model<-glm(happy~years+stipend, family=binomial(link="logit"))
summary(logit.model)

# figure out marginal effect of moving years from 2 to 3, 
# holding stipend at mean
mean.stipend<-mean(stipend)
predict.data<-data.frame(years=c(2,3), stipend=rep(mean.stipend,2))
change<-predict(logit.model, newdata=predict.data, type="response")
change[2]-change[1]
#get the coefficients and VCV matrix
coefficients(logit.model)
vcov(logit.model)

# how about a probit?
probit.model<-glm(happy~years+stipend, family=binomial(link="probit"))
summary(probit.model)

# clear everything out again
detach(my.data.set)
rm(list=ls())





##############################
# Basic programming features:
# Scalars and Vectors
##############################


# create a scalar
a<-5

# display it (you can also see it in the "Workspace" pane)
a

# create a vector
b<-c(1,2,3,4,5)     # c() means "concatenate"

# What's the product of a*b?
a*b

# what's the product of two vectors?
d<-c(2,3,2,1,2)
b*d

# what's the sum/difference of two vectors?
b+d
b-d

# What's the third element of b?
b[3]
b[c(1,5)]
b[2:4]
d[2:4]






##############################
# Basic programming features:
# Matrices
#####################

# enter the matrix called A
A<-matrix(c(1,2,3,4,5,6), nrow=3, ncol=3)

# show that matrix
A

# note the difference if we tell to enter by row
AA<-matrix(c(1,2,3,4,5,6), nrow=3, ncol=3, byrow=T)
AA

# element-by-element sum and product of matrices
AA+A
AA*A

# "true" matrix multiplication
AA%*%A

# enter a simple vector x
x<-c(1,2,3,4,5,6)

# change the vector into a matrix
AA<-matrix(x, nrow=3, ncol=3, byrow=T)
AA

# R thinks that a row vector is DIFFERNT than a 1xk matrix
AA<-matrix(x, nrow=1, byrow=T)
AA

is.matrix(x)
is.matrix(AA)

# extract the first row vector
A[1,]

# extract the first column vector
A[,1]

# extract the middle element 
A[2,2]

# extract a sub-matrix from matrix A, call it B
B<-A[2:3,2:3]
B

D<-matrix(c(1,2,6,2,4,2), ncol=1)
AA*D  # doesn't work
AA%*%D  # 1x1 scalar
D%*%AA  # 6x6 matrix


##############################
# Basic programming features:
# Arrays
#####################

X<-array(data=runif(36), dim=c(3,3,4))
X

X[1,1,3]
X[3,1,1]
X[,,2]


##############################
# Basic programming features:
# Lists
#####################

Y<-list(one=X[,,1], two=X[,,2], three=X[,,3])

Y$one
Y$two

Y<-list(one=X[,,1], two=X[,,2], three=c("convergence achieved"))


##############################
# Basic programming features:
# Random Variable Generators
#####################

# the uniform
?runif
runif(100)
runif(100, min=-10, max=10)
dunif(5, min=-10, max=10)

# the normal
?rnorm
rnorm(100)
rnorm(100, mean=-2, sd=3)
pnorm(-1.96)

# the multivariate normal
library(mvtnorm)
X<-rmvnorm(1000, mean=c(0,1), sigma=matrix(data=c(1,0.5,0.5,1), ncol=2, nrow=2, byrow=T))
plot(X[,1], X[,2])












##############################
# Basic programming features:
# Control Conditions
###############

# for loops
# the fibonacci sequence
a<-c(1,1)
for(i in 3:50){
  
  a[i]<-a[i-1]+a[i-2]
  
}

# if conditions
# dichotomize a variable
a<-runif(100, min=-5, max=5)  # draw from the uniform distribution
b<-c()                        # empty vector
for(i in 1:length(a)){
  
  if(a[i]>0){
    b[i]<-1
  }else{
    b[i]<-0
  }
  
}
# ...or, you could just do...
bb<-c()
bb<-as.numeric(a>0)










##############################
# Basic programming features:
# Functions
###############

myfunction<-function(a,b){
  
  out<-a^b
  return(out)
  
}

myfunction(2,2)





myfunction.two<-function(A){
  
  out<-A[1]^A[2]
  return(out)
  
}

myfunction.two(c(2,2))


# apply (like a for loop)
?apply

sample.mat<-matrix(data=runif(100), nrow=50, ncol=2)
apply(X=sample.mat, MARGIN=1, FUN=myfunction.two)

store<-c()
for(i in 1:dim(sample.mat)[1]){
  
  store[i]<-myfunction.two(sample.mat[i,])
  
}
store
